@@ -6,6 +6,7 @@ Created on 2 oct. 2014
 
 import re
 import sys
+import os
 
 from distutils.version import LooseVersion          # @UnusedImport
 from distutils.errors import DistutilsError
@@ -165,6 +166,9 @@ def pip_install_package(package,directory=None,pip=None):
                 
     args = ['install']
     
+    if 'http_proxy' in os.environ:
+        args.append('--proxy=%s' % os.environ['http_proxy'])
+        
     if directory is not None:
         args.append('--target=%s' % directory)
     
