@@ -7,7 +7,11 @@ from obitools.ecopcr import EcoPCRDBFile
 from obitools.utils import universalOpen
 from obitools.utils import ColumnFile
 import math
-from collections import Counter
+try:
+    from collections import Counter
+except ImportError:
+    from obitools.collections import Counter
+
 
 class Taxonomy(object):
     def __init__(self):
@@ -24,7 +28,14 @@ class Taxonomy(object):
         self._genusidx   = self._ranks.index('genus')
         self._familyidx   = self._ranks.index('family')
         self._orderidx   = self._ranks.index('order')
-        self._nameidx=dict((x[0],x[2]) for x in self._name)
+        
+        self._nameidx = {}
+        for x in self._name :
+            if x[0] not in self._nameidx :
+                self._nameidx[x[0]] = [x[2]]
+            else :
+                self._nameidx[x[0]].append(x[2])
+        
         self._nameidx.update(dict((x[0],x[2]) for x in self._preferedName))
         self._preferedidx=dict((x[2],x[1]) for x in self._preferedName)
         
@@ -45,8 +56,11 @@ class Taxonomy(object):
         return self.findTaxonByIdx(self.findIndex(taxid))
         
     def findTaxonByName(self,name):
-        return self._taxonomy[self._nameidx[name]]
-    
+        taxa = []
+        for i in self._nameidx[name] :
+            taxa.append(self._taxonomy[i])
+        return taxa
+        
     def findRankByName(self,rank):
         try:
             return self._ranks.index(rank)
@@ -54,7 +68,10 @@ class Taxonomy(object):
             return None
         
     def __contains__(self,taxid):
-        return self.findTaxonByTaxid(taxid) is not None
+        try:
+            return self.findTaxonByTaxid(taxid) is not None
+        finally:
+            return False
     
         
 
@@ -68,7 +85,11 @@ class Taxonomy(object):
 
     def subTreeIterator(self, taxid):
         "return subtree for given taxonomic id "
-        idx = self.findTaxonByTaxid(taxid)
+        
+        for t in xrange(len(self._taxonomy)) :
+            if self._taxonomy[t][0] == taxid :
+                idx = t
+        
         yield self._taxonomy[idx]
         for t in self._taxonomy:
             if t[2] == idx:
@@ -78,7 +99,7 @@ class Taxonomy(object):
     def parentalTreeIterator(self, taxid):
         """
            return parental tree for given taxonomic id starting from
-           first ancester to the root.
+           first ancestor to the root.
         """
         taxon=self.findTaxonByTaxid(taxid)
         if taxon is not None:
@@ -208,6 +229,7 @@ class Taxonomy(object):
         self._taxonomy.append((newtaxid,rankid,a.pop(),groupname))
         for x in t:
             x[2]=newidx
+       
             
     def addLocalTaxon(self,name,rank,parent,minimaltaxid=10000000):
         newtaxid = minimaltaxid if (self._bigestTaxid < minimaltaxid) else self._bigestTaxid+1
@@ -218,13 +240,19 @@ class Taxonomy(object):
         self._taxonomy.append(tx)
         newidx=len(self._taxonomy)-1
         self._name.append((name,'scientific name',newidx))
-        self._nameidx[name]=newidx
+        
+        if name not in self._nameidx :
+            self._nameidx[name]=[newidx]
+        else :
+            self._nameidx[name].append(newidx)
+               
         self._index[newtaxid]=newidx
 
         self._bigestTaxid=newtaxid
         
         return newtaxid
     
+    
     def removeLocalTaxon(self,taxid):
         raise NotImplemented
         txidx = self.findIndex(taxid)
@@ -235,18 +263,17 @@ class Taxonomy(object):
         for t in self._taxonomy:
             if t[2] == txidx:
                 self.removeLocalTaxon(t[0])
-                
-        
-        
         
         return taxon
     
+    
     def addPreferedName(self,taxid,name):
         idx = self.findIndex(taxid)
         self._preferedName.append(name,'obi',idx)
         self._preferedidx[idx]=name
         return taxid
 
+
 class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
     '''
     A taxonomy database class
@@ -268,7 +295,7 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
         self._preferedNamesFile =  "%s.pdx" % self._path
         self._aliasFile =  "%s.adx" % self._path
         
-        print >> sys.stderr,"Reading binary taxonomy database...",
+        print >> sys.stderr,"Reading binary taxonomy database..."
         
         self.__readNodeTable()
         
@@ -302,10 +329,10 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
             
         try :
             lt=0
-            for record in self._ecoRecordIterator(self._localTaxonFile):
+            for record in self._ecoRecordIterator(self._localTaxonFile,noError=True):
                 lrecord = len(record)
                 lnames  = lrecord - 16
-                (taxid,rankid,parentidx,nameLength,name)=struct.unpack('> I I I I %ds' % lnames, record)
+                (taxid,rankid,parentidx,nameLength,name)=struct.unpack('> I I I I %ds' % lnames, record)  # @UnusedVariable
                 lt+=1
                 yield  (taxid,rankid,parentidx,name,'local')
             print >> sys.stderr, " [INFO : Local taxon file found] : %d added taxa" % lt
@@ -317,7 +344,7 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
             yield  record
     
     def __ecoAliasIterator(self):
-        for record in self._ecoRecordIterator(self._aliasFile):
+        for record in self._ecoRecordIterator(self._aliasFile,noError=True):
             (taxid,index) = struct.unpack('> I i',record)
             yield taxid,index
             
@@ -345,7 +372,6 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
         except:
             print >> sys.stderr, " [INFO : Taxon alias file not found] "
             index={}
-            i = 0;
             buildIndex=True
             
         localtaxon=0
@@ -360,8 +386,8 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
             i+=1
             
                 
-        print  >> sys.stderr,"Taxonomical tree read",
-        return taxonomy, index,localtaxon
+        print  >> sys.stderr,"Taxonomical tree read"
+        return taxonomy, index, localtaxon
 
     def __readNodeTable(self):
         self._taxonomy, self._index, self._localtaxon= self.__ecoTaxonomyIndex()
@@ -376,10 +402,10 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
                 
         try :
             self._preferedName = [(x[0],'obi',x[2]) 
-                                  for x in self.__ecoNameIterator(self._preferedNamesFile)]
-            print >> sys.stderr, " [INFO : Prefered taxon name file found] : %d added taxa" % len(self._preferedName)
+                                  for x in self.__ecoNameIterator(self._preferedNamesFile,noError=True)]
+            print >> sys.stderr, " [INFO : Preferred taxon name file found] : %d added taxa" % len(self._preferedName)
         except:
-            print >> sys.stderr, " [INFO : Prefered taxon name file not found]"
+            print >> sys.stderr, " [INFO : Preferred taxon name file not found]"
             self._preferedName = []
             
                 
@@ -407,8 +433,13 @@ class TaxonomyDump(Taxonomy):
         print >>sys.stderr,"Adding deleted taxid..."
         for taxid in self._deletedNodeIterator('%s/delnodes.dmp' % taxdir):
             self._index[taxid]=None
-            
-        self._nameidx=dict((x[0],x[2]) for x in self._name)
+        
+        self._nameidx = {}
+        for x in self._name :
+            if x[0] not in self._nameidx :
+                self._nameidx[x[0]] = [x[2]]
+            else :
+                self._nameidx[x[0]].append(x[2])
 
             
     def _taxonCmp(t1,t2):
